﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Data.Command {
  [StorableClass]
  public class DeleteColumnCommand : ColumnGroupCommandWithAffectedColumnsBase {
    private List<ColumnBase> deletedColumns;
    private List<int> oldSortedColumnIndices;

    private DeleteColumnCommand()
      : base(null, string.Empty, null) {
      deletedColumns = new List<ColumnBase>();
    }

    public DeleteColumnCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      deletedColumns = new List<ColumnBase>();
    }

    public override void Execute() {
      base.Execute();
      this.oldSortedColumnIndices = new List<int>(ColumnGroup.SortedColumnIndexes);
      ColumnBase column;
      foreach (int columnIndex in AffectedColumns.Reverse()) {
        column = ColumnGroup.GetColumn(columnIndex);
        deletedColumns.Insert(0, column);
        ColumnGroup.RemoveColumn(column);
        if (column.SortOrder != SortOrder.None)
          ColumnGroup.SortedColumnIndexes.Remove(columnIndex);
      }
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      for (int i = 0; i < AffectedColumns.Length; i++)
        ColumnGroup.InsertColumn(AffectedColumns[i], deletedColumns[i]);

      deletedColumns.Clear();
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndices;
      oldSortedColumnIndices = null;
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override string Description {
      get { return "Columns deleted"; }
    }
  }
}
